#include "BensleyTones.h"
#include <TimerOne.h>

// Floppy drive music take 1!

//#define DriveSelect 5 // IDE 14 : Not Used : Grounded 
//#define f1Direction 4 // IDE 18, LOW = move forward, HIGH = move backward
//#define f1HeadStep 5 // IDE 20
//#define f2Direction 2
//#define f2HeadStep 3

int aryPinDirection[] = {2, 4};
int aryPinHeadStep[] = {3, 5};

/*
* BMP = 120
* Thats 1 note every 500ms
* half note = 250ms
* quater note = 125ms
* 
*/

// Calculating note lengths using 500ms as 1 beat:
// Here we are "padding notes", so that when the drive oscilates at 100Hz the note would
// be over quicker than if it where oscilating at 50Hz, so we use are pretent scale value
// and calculate the number of motor steps needed to play a note given its delay 
// within a given note length
    /*
    Normal Notes:
    250ms beat = 500 / 2 = 250
    44000 / 69 = 6376, *2 = 12734
    12734/1000 = 12.73
    250/12.73 = 19
    
    Rests:
    500ms beat = 500 / 1 = 500
    440000 / 1.76 = 250000, *2 = 500000
    500000 / 1000 = 500
    500 / 500 = 1
    
    Rests - 2:
    250ms beat = 500 / 2 = 250
    440000 / 3.52 = 125000, *2 = 250000
    250000 / 1000 = 250
    250 / 250 = 1
    */

const int beat = 500;
const long scale = 440000;
const int timerDelay = 100;

float aryNotes0[] = {  // Drive 1 High - This has to be a float if using any of the REST_x notes
  // Bars 1-4
  NOTE_B1,NOTE_CS2,NOTE_D2,NOTE_E2,NOTE_FS2,NOTE_D2,NOTE_FS2,
  NOTE_F2,NOTE_CS2,NOTE_F2,NOTE_E2,NOTE_C1,NOTE_E2,
  NOTE_B1,NOTE_CS2,NOTE_D2,NOTE_E2,NOTE_FS2,NOTE_D2,NOTE_FS2,NOTE_B2,
  NOTE_A2,NOTE_FS2,NOTE_D2,NOTE_FS2,NOTE_A2,REST_WHOLE,
  // Bars 5-8
  NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,
  NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,
  NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,
  NOTE_D2,REST_HALF,NOTE_A2,REST_HALF,NOTE_D2,REST_HALF,NOTE_A2,REST_HALF,
  // Bars 9 to 12
  NOTE_FS2,NOTE_GS2,NOTE_AS2,NOTE_B2,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST_HALF,
  NOTE_D3,NOTE_AS2,NOTE_D3,NOTE_CS3,NOTE_AS2,NOTE_CS3,
  NOTE_FS2,NOTE_GS2,NOTE_AS2,NOTE_B2,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST_HALF,
  NOTE_D3,NOTE_AS2,NOTE_D3,NOTE_CS3,REST_WHOLE,
  // Bars 13 to 16
  NOTE_FS2,NOTE_CS3,NOTE_FS2,NOTE_AS2,
  NOTE_D2,NOTE_AS2,NOTE_FS2,NOTE_AS2,
  NOTE_FS2,NOTE_CS3,NOTE_FS2,NOTE_AS2,
  NOTE_D2,NOTE_AS2,NOTE_FS2,NOTE_AS2
};

// The note duration is a faction of a whole beat so we can have crotchets and quavers etc
// 1 / 2 = half beat, 1 / 4 = quater beat
// this has to be a float if we want to double up, as its used in a division,
// 1 / 0.5 = 2 for 2 beats long
int aryDuration0[] = { // Drive 1 High
  // Bars 1-4
  2,2,2,2,2,2,1,
  2,2,1,2,2,1,
  2,2,2,2,2,2,2,2,
  2,2,2,2,1,1,    // 27th Note
  // Bars 5-8
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,  //  59th Note
  // Bars 9-12
  2,2,2,2,2,2,2,2,
  2,2,1,2,2,1,
  2,2,2,2,2,2,2,2,
  2,2,1,1,1,        //  86th Note
  // Bars 13-16
  1,1,1,1,
  1,1,1,1,
  1,1,1,1,
  1,1,1,1          //  102nd Note
};

float aryNotes1[] = { // Drive 1 Low
  // Bars 1-4
  NOTE_B0,NOTE_CS1,NOTE_D1,NOTE_E1,NOTE_FS1,NOTE_D1,NOTE_FS1,
  NOTE_F1,NOTE_CS1,NOTE_F1,NOTE_E1,NOTE_C1,NOTE_E1,
  NOTE_B0,NOTE_CS1,NOTE_D1,NOTE_E1,NOTE_FS1,NOTE_D1,NOTE_FS1,NOTE_B1,
  NOTE_A1,NOTE_FS1,NOTE_D1,NOTE_FS1,NOTE_A1,REST_WHOLE,
  // Bars 5-8
  NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,
  NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,
  NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,
  NOTE_D1,REST_HALF,NOTE_A1,REST_HALF,NOTE_D1,REST_HALF,NOTE_A1,REST_HALF,
  // Bars 9-12
  NOTE_FS1,NOTE_GS1,NOTE_AS1,NOTE_B1,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST_HALF,
  NOTE_D2,NOTE_AS1,NOTE_D2,NOTE_CS2,NOTE_AS1,NOTE_CS2,
  NOTE_FS1,NOTE_GS1,NOTE_AS1,NOTE_B1,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST_HALF,
  NOTE_D2,NOTE_AS1,NOTE_D2,NOTE_CS2,REST_WHOLE,
  // Bars 13-16
  NOTE_FS1,NOTE_CS2,NOTE_FS1,NOTE_AS1,
  NOTE_D1,NOTE_AS1,NOTE_FS1,NOTE_AS1,
  NOTE_FS1,NOTE_CS2,NOTE_FS1,NOTE_AS1,
  NOTE_D1,NOTE_AS1,NOTE_FS1,NOTE_AS1
};

int aryDuration1[] = { // Drive 1 Low
  // Bars 1-4
  2,2,2,2,2,2,1,
  2,2,1,2,2,1,
  2,2,2,2,2,2,2,2,
  2,2,2,2,1,1,    // 27th Note
  // Bars 5-8
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,  //  59th Note
  // Bars 9-12
  2,2,2,2,2,2,2,2,
  2,2,1,2,2,1,
  2,2,2,2,2,2,2,2,
  2,2,1,1,1,        //  86th Note
  // Bars 13-16
  1,1,1,1,
  1,1,1,1,
  1,1,1,1,
  1,1,1,1          //  102nd Note
};

float aryNotes2[] = { // Drive 2 High
  // Bars 1-4
  NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,
  NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,
  NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,NOTE_B1,REST_HALF,NOTE_FS2,REST_HALF,
  NOTE_D2,REST_HALF,NOTE_A2,REST_HALF,NOTE_D2,REST_HALF,NOTE_A2,REST_HALF,
  // Bars 5-8
  NOTE_B1,NOTE_CS2,NOTE_D2,NOTE_E2,NOTE_FS2,NOTE_D2,NOTE_FS2,REST_HALF,
  NOTE_F2,NOTE_CS2,NOTE_F2,REST_HALF,NOTE_E2,NOTE_B1,NOTE_E2,REST_HALF,
  NOTE_B1,NOTE_CS2,NOTE_D2,NOTE_E2,NOTE_FS2,NOTE_D2,NOTE_FS2,NOTE_B2,
  NOTE_A2,NOTE_FS2,NOTE_D2,NOTE_FS2,NOTE_A2,REST_WHOLE,
  // Bars 9-12
  NOTE_FS2,REST_HALF,NOTE_CS3,REST_HALF,NOTE_FS2,REST_HALF,NOTE_AS2,REST_HALF,
  NOTE_D2,REST_HALF,NOTE_AS2,REST_HALF,NOTE_FS2,REST_HALF,NOTE_AS2,REST_HALF,
  NOTE_FS2,REST_HALF,NOTE_CS3,REST_HALF,NOTE_FS2,REST_HALF,NOTE_AS2,REST_HALF,
  NOTE_D2,REST_HALF,NOTE_AS2,REST_HALF,NOTE_FS2,REST_HALF,NOTE_AS2,REST_HALF,
  // Bars 13-16
  NOTE_FS2,NOTE_GS2,NOTE_AS2,NOTE_B2,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST_HALF,
  NOTE_D3,NOTE_AS2,NOTE_D3,REST_HALF,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST_HALF,
  NOTE_FS2,NOTE_GS2,NOTE_AS2,NOTE_B2,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST_HALF,
  NOTE_D3,NOTE_AS2,NOTE_D3,REST_HALF,NOTE_CS3,REST_HALF,REST_WHOLE
};

int aryDuration2[] = { // Drive 2 High
  // Bars 1-4
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,    // 32nd Note
  // Bars 5-8
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,1,1,        // 62nd Note
  // Bars 9-12
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,    // 95th Note
  // Bars 13-16
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,1       // 126th Note
};

float aryNotes3[] = { // Drive 2 Low
  // Bars 1-4
  NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,
  NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,
  NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,NOTE_B0,REST_HALF,NOTE_FS1,REST_HALF,
  NOTE_D1,REST_HALF,NOTE_A1,REST_HALF,NOTE_D1,REST_HALF,NOTE_A1,REST_HALF,
  // Bars 5-8
  NOTE_B0,NOTE_CS1,NOTE_D1,NOTE_E1,NOTE_FS1,NOTE_D1,NOTE_FS1,REST_HALF,
  NOTE_F1,NOTE_CS1,NOTE_F1,REST_HALF,NOTE_E1,NOTE_B0,NOTE_E1,REST_HALF,
  NOTE_B0,NOTE_CS1,NOTE_D1,NOTE_E1,NOTE_FS1,NOTE_D1,NOTE_FS1,NOTE_B1,
  NOTE_A1,NOTE_FS1,NOTE_D1,NOTE_FS1,NOTE_A1,REST_HALF,
  // Bars 9-12
  NOTE_FS1,REST_HALF,NOTE_CS2,REST_HALF,NOTE_FS1,REST_HALF,NOTE_AS1,REST_HALF,
  NOTE_D1,REST_HALF,NOTE_AS1,REST_HALF,NOTE_FS1,REST_HALF,NOTE_AS1,REST_HALF,
  NOTE_FS1,REST_HALF,NOTE_CS2,REST_HALF,NOTE_FS1,REST_HALF,NOTE_AS1,REST_HALF,
  NOTE_D1,REST_HALF,NOTE_AS1,REST_HALF,NOTE_FS1,REST_HALF,NOTE_AS1,REST_HALF,
  // Bars 13-16
  NOTE_FS1,NOTE_GS1,NOTE_AS1,NOTE_B1,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST_HALF,
  NOTE_D2,NOTE_AS1,NOTE_D2,REST_HALF,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST_HALF,
  NOTE_FS1,NOTE_GS1,NOTE_AS1,NOTE_B1,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST_HALF,
  NOTE_D2,NOTE_AS1,NOTE_D2,REST_HALF,NOTE_CS2,REST_HALF,REST_WHOLE
};

int aryDuration3[] = { // Drive 2 Low
  // Bars 1-4
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,    // 32nd Note
  // Bars 5-8
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,1,1,        // 62nd Note
  // Bars 9-12
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,    // 95th Note
  // Bars 13-16
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,1       // 126th Note
};

const int driveCount = 2;
int driveIndex;
int driveStart = 0;

int aryNoteIndex[4] = {0, 0, 0, 0};
int aryNoteCount[4] = {102, 102, 124}; //{27, 27, 32, 32};
int aryNoteLength[4] = {0, 0, 0, 0};
long aryNoteDelay[4] = {0, 0, 0, 0};
int aryHighLow[4] = {0, 0, 0, 0};
float aryWaiting[4] = {0, 0, 0, 0};

void centerHead() {
  digitalWrite(aryPinHeadStep[driveIndex], HIGH);
  digitalWrite(aryPinDirection[driveIndex], HIGH);
  // Move header to start position from wherever it is, 80 steps at ~261 step/s
  for(int i=0;i<80;i++) {
    digitalWrite(aryPinHeadStep[driveIndex],LOW);
    delay(2);
    digitalWrite(aryPinHeadStep[driveIndex],HIGH); 
    delay(2);
  }
  delay(2);
  digitalWrite(aryPinDirection[driveIndex], LOW); // Change direction, and move then back 40 steps so we are centered (the step range is 80 wide)
  for(int i=0;i<40;i++) {
    digitalWrite(aryPinHeadStep[driveIndex],LOW);
    delay(2);
    digitalWrite(aryPinHeadStep[driveIndex],HIGH); 
    delay(2);
  }
}

void setupNote(){
  if(aryNoteIndex[driveIndex]<aryNoteCount[driveIndex]) {
    if(driveIndex==0) {
      aryNoteDelay[driveIndex] = (scale/aryNotes0[aryNoteIndex[driveIndex]]);
      if(aryNoteDelay[driveIndex]==250000 || aryNoteDelay[driveIndex]==125000) {
        aryNoteLength[driveIndex] = 1;
      }else{
        int x = beat/aryDuration0[aryNoteIndex[driveIndex]];
        int y = scale/aryNotes0[aryNoteIndex[driveIndex]];
        y = y * 2;
        y = y / 1000;
        aryNoteLength[driveIndex] = x / y;
        
        //aryNoteLength[driveIndex] = ((beat/aryDuration0[aryNoteIndex[driveIndex]])/((2*(scale/aryNotes0[aryNoteIndex[driveIndex]]))/1000));
      }  
    }
    if (driveIndex==1) {
      aryNoteDelay[driveIndex] = (scale/aryNotes1[aryNoteIndex[driveIndex]]);
      if(aryNoteDelay[driveIndex]==250000 || aryNoteDelay[driveIndex]==125000) {
        aryNoteLength[driveIndex] = 1;
      }else{
        int x = beat/aryDuration1[aryNoteIndex[driveIndex]];
        int y = scale/aryNotes1[aryNoteIndex[driveIndex]];
        y = y * 2;
        y = y / 1000;
        aryNoteLength[driveIndex] = x / y;
      //aryNoteLength[driveIndex] = ((beat/aryDuration1[aryNoteIndex[driveIndex]])/((2*(scale/aryNotes1[aryNoteIndex[driveIndex]]))/1000));
      }  
    }
  aryNoteIndex[driveIndex]++;
  aryHighLow[driveIndex] = 0;
  aryWaiting[driveIndex] = 0;
  }else{
    aryHighLow[driveIndex]=3; // To stop the motor loop
  }
}

void setup() {
  delay(500);
  for(driveIndex=driveStart;driveIndex<driveCount;driveIndex++) { // Initialise, and center the motor's of all drives
    pinMode(aryPinDirection[driveIndex], OUTPUT); 
    pinMode(aryPinHeadStep[driveIndex], OUTPUT);
    centerHead();
    setupNote();
  }
  delay(1000);
  Timer1.initialize(timerDelay); // 1ms
  Timer1.attachInterrupt(play);
  
}

void play() {
  for(driveIndex=driveStart;driveIndex<driveCount;driveIndex++) {
    if(aryWaiting[driveIndex]<=0) { // If 0 (start of note)
      if(aryHighLow[driveIndex]==0) {
        digitalWrite(aryPinDirection[driveIndex], HIGH);
        digitalWrite(aryPinHeadStep[driveIndex],LOW);
        digitalWrite(aryPinHeadStep[driveIndex],HIGH);
        aryHighLow[driveIndex]=1;
        aryWaiting[driveIndex] = aryNoteDelay[driveIndex];
      }else if(aryHighLow[driveIndex]==1){
        digitalWrite(aryPinDirection[driveIndex], LOW);
        digitalWrite(aryPinHeadStep[driveIndex],LOW);
        digitalWrite(aryPinHeadStep[driveIndex],HIGH);
        aryHighLow[driveIndex]=2;
        aryWaiting[driveIndex] = aryNoteDelay[driveIndex];
      }else if(aryHighLow[driveIndex]==2) { // End of the note!
        aryNoteLength[driveIndex]--;
        aryHighLow[driveIndex]=0;
        if(aryNoteLength[driveIndex]==0) {
          setupNote();
        }
      }
    }else{
      aryWaiting[driveIndex]=aryWaiting[driveIndex]-timerDelay;
    }
  }
}

void loop() {

}
